//----------------------------------------------------------------
//
// Copyright 1998-1999 (c) RC Module.
// NeuroChip NM6403.
//
// nc_int.h
// Header file for low level library.
// Contain declaration of timer, comm. port and interrupt managing function
// ver 1.0.
//
// by Alexey A. Chelubeev
//
//----------------------------------------------------------------

#ifndef _NC_INT_H
#define _NC_INT_H

#include <stdlib.h>


    // Interrupt masks. Can be combined by bit operaitons.

const int MIALL = 0x3ff0;       // All interrupts.
const int MIT0 = 0x2000;        // Interrupt from timer T0 in zero.
const int MIOVERFLOW = 0x1000;  // Arithmetic overflow in scalar processor.
const int MIVECT = 0x800;       // Invalid vector command interrupt.
const int MIOUTSIDE = 0x400;    // External interrupt.
const int MICOM1IN = 0x200;     // Interrupt on end of input on comm. port 1.
const int MICOM0IN = 0x100;     // Interrupt on end of input on comm. port 0.
const int MICOM1OUT = 0x80;     // Interrupt on end of output on comm. port 1.
const int MICOM0OUT = 0x40;     // Interrupt on end of output on comm. port 0.
const int MIT1 = 0x20;          // Interrupt from timer T1 in zero.
const int MISTEP = 0x10;        // Step-by-step interrupt.


#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

    //--------------------------------------------
    //
    // Function for TIMER pin out control.
    //
    //--------------------------------------------

    // Constants for TIMER pin out control.
    // Used by TimerPinOut function.
enum TIMER_PIN_MODE {
    TP_TRUE           = 0,    //  
    TP_FALSE          = 1,    //  
    TP_INT_T0_TIMER   = 2,    // INT_T0 xor TIMER
    TP_INT_T1_TIMER   = 3,    // INT_T1 xor TIMER
    TP_INT_T0         = 4,    // INT_T0
    TP_INT_T1         = 5,    // INT_T1
    TP_NOT_INT_T0     = 6,    // not INT_T0
    TP_NOT_INT_T1     = 7     // not INT_T1
};

    // Disable TIMER pin out.
size_t TimerPinHalt ( void );

    // Enable TIMER pibn out.
size_t TimerPinRelease ( void );

    // Change TIMER pin output .
    // Defined by constants TP_XXXX.
size_t TimerPinOut ( TIMER_PIN_MODE det_out );


    //--------------------------------
    //
	// Timer functions.
    //
    //--------------------------------

    // Set timer to work in single repeat mode.
    // Timer count to zero, raise unterrupt and stop.
size_t IntptTimerMode ( int timer_no );

    // Set timer to work in loop mode.
    // Timer count to zero, raise interrupt, restore initial
    // timer value and count again.
size_t UnIntptTimerMode ( int timer_no );

    // Enable timer work, dependent from bit 19 (17) of PSWR.
size_t ClearTimerCtrl ( int timer_no );

    // Enable timer work, independent from bit 19 (17) of PSWR.
size_t SetTimerCtrl ( int timer_no );

    // Set timer initila value.
    // Return previous timer value.
size_t SetTimerCounter ( int timer_no, size_t CounterValue );

    // Return current timer value.
size_t GetTimerCounter ( int timer_no );



    //------------------------------------------------
    //
    // Communication ports function.
    //
    //------------------------------------------------

    // Switch comm. port to program init mode.
size_t CpPrgrmInit ( int ch_no );

    // Switch comm. port to first word init mode.
size_t CpFrstWordInit ( int ch_no );

    // Suspend input from port ch_no.
size_t CpInHalt ( int ch_no );

    // Resume input from port ch_no.
size_t CpInRelease ( int ch_no );

    // Suspend output to port ch_no.
size_t CpOutHalt ( int ch_no  );
    // Resume output to port ch_no.
size_t CpOutRelease ( int ch_no );

    // Send BufSize (64-bit) words from address BufAddr to port ch_no.
void CpSendMsg ( int ch_no, size_t BufSize, void * BufAddr );

    // Receive BufSize (64-bit) words from port ch_no to address BufAddr.
void CpReceiveMsg ( int ch_no, size_t BufSize, void * BufAddr);

    // Return input channel counter for port ch_no.
size_t GetCpInCount ( int ch_no );

    // Return optput channel counter for port ch_no.
size_t GetCpOutCount ( int ch_no );



    //-----------------------------------
    //
    // Interrupt handling functions.
    //
    //-----------------------------------


    // Interrupt handler function type.
typedef void ( * InterruptFunc ) ( void );

    // Interrupt numbers.
enum InterruptNum {
  INTT0 = 1,        // Timer T0 count to zero.
  INTOVERFLOW = 2,  // Arithmetic overflow in scalar processor.
  INTVECT = 3,      // Invalid vector command interrupt.
  INTOUTSIDE = 4,   // External interrupt.
  INTCOM1IN = 5,    // End of input on comm. port 1.
  INTCOM0IN = 6,    // End of input on comm. port 0.
  INTCOM1OUT = 7,   // End of output on comm. port 1.
  INTCOM0OUT = 8,   // End of output on comm. port 0.
  INTT1 = 9,        // Timer 1 count to zero.
  INTSTEP = 10      // Step-by-step interrupt.
};


    // Set 'function' as interrupt 'num' handler.
    // Return old handler.
InterruptFunc SetInterrupt ( 
        InterruptNum num,
		InterruptFunc function 
    );

    // Return current interrupt 'num' handler.
InterruptFunc GetInterrupt ( InterruptNum num );

    // Clear interrupt 'num' handler.
InterruptFunc ClearInterrupt ( InterruptNum num );


    // Enable interrupts in mask.
void EnableInterrupts ( int mask );

    // Disbale interrupts in mask.
void DisableInterrupts ( int mask );

    // Return current interrupt mask.
    // Bit in mask set ==> interrupt enabled.
int GetInterruptMask ( void );


#ifdef __cplusplus
}
#endif // __cplusplus

#endif // _NC_INT_H
